#!/usr/bin/wish -f
# A TCL script to make various plots via gnuplot for the weak.oct program.

proc nokey {} {
    exec <@stdin >@stdout echo {set nokey};
}

proc plot1var {file c lim mn mx} {
    nokey
    if {$lim == 0} {
	set strng "plot \"$file\" using 0:$c with lines"
    } else {
	set strng "plot \[$mn:$mx\] \"$file\" using 0:$c with lines"
    }
    exec <@stdin >@stdout echo $strng
}

proc plot2var {file c1 c2 lim mn mx} {
    nokey
    if {$lim == 0} {
	set strng "plot \"$file\" using 0:$c1, \"$file\" using 0:$c2"
    } else {
      set strng "plot \[$mn:$mx\] \"$file\" using 0:$c1, \"$file\" using 0:$c2"
    }
    exec <@stdin >@stdout echo $strng
}

proc plot1vs2varlines {file c1 c2 lim mn mx} {
    nokey
    if {$lim == 0} {
	set strng "plot \"$file\" using $c1:$c2 with lines"
    } else {
	set strng "plot \[:\] \[$mn:$mx\] \"$file\" using $c1:$c2 with lines"
    }
    exec <@stdin >@stdout echo $strng
}

proc plot1vs2var {file c1 c2 lim mn mx} {
    nokey
    if {$lim == 0} {
	set strng "plot \"$file\" using $c1:$c2"
    } else {
	set strng "plot \[:\] \[$mn:$mx\] \"$file\" using $c1:$c2"
    }
    exec <@stdin >@stdout echo $strng
}

proc splott {file c1 c2 c3 lim mn mx} {
    nokey
    if {$lim == 0} {
	set strng "splot \"$file\" using (\$$c1/2/pi):$c2:$c3"
    } else {
	set strng "set view $mn, $mx\nsplot \"$file\" using (\$$c1/2/pi):$c2:$c3"
    }
    exec <@stdin >@stdout echo $strng
}

proc splotp {file c1 c2 c3 lim mn mx} {
    nokey
    if {$lim == 0} {
	set strng "splot \"$file\" using $c1:$c2:$c3"
    } else {
	set strng "set view $mn, $mx\nsplot \"$file\" using $c1:$c2:$c3"
    }
    exec <@stdin >@stdout echo $strng
}

proc calculate {rho n ns x0 xp0 y0 yp0 z0 d0 nt} {
    set strng "set title \"n=$n, rho=$rho, x0=$x0, xp0=$xp0, y0=$y0, yp0=$yp0"
    exec <@stdin >@stdout echo $strng
    set strng "rho=$rho;\nn=$n;\nx0=$x0;\nxp0=$xp0;\ny0=$y0;\nyp0=$yp0;\nz0=$z0;\ndelta0=$d0;\nnturns=$nt;\nnsteps=$ns;\n"
    set ff [open "weak.params" w]
    puts $ff $strng
    close $ff
    set strng "!cat weak.params weak.oct|octave"
    exec <@stdin >@stdout echo $strng
}
#---------------------------------------
set rho 1
set n 0.5
set nsteps 100
set nturns 1
set x0 0.001
set xp0 0
set y0 0
set yp0 0.0002
set z0 0
set delta0 0
#---------------------------------------
frame .xvst
pack .xvst -fill x

checkbutton .xvst.lim -text limits -variable xvstlim -relief flat

entry .xvst.lim_min -width 10 -relief sunken -bd 2 -textvariable xvst_lim_min
label .xvst.to -text "to"
entry .xvst.lim_max -width 10 -relief sunken -bd 2 -textvariable xvst_lim_max

button .xvst.plot -text "x vs theta" -command {upvar 0 xvstlim lim; \
	upvar 0 xvst_lim_min mn;\
	upvar 0 xvst_lim_max mx;\
	set strng "set xlabel \"theta \[radian\]\"\nset ylabel \"x \[m\]\"\n";\
	exec <@stdin >@stdout echo $strng;\
	plot1vs2varlines weak.dat 1 3 $lim $mn $mx}

pack .xvst.plot .xvst.lim .xvst.lim_min .xvst.to .xvst.lim_max -side left -fill x
#---------------------------------------
frame .yvst
pack .yvst -fill x

checkbutton .yvst.lim -text limits -variable yvstlim -relief flat

entry .yvst.lim_min -width 10 -relief sunken -bd 2 -textvariable yvst_lim_min
label .yvst.to -text "to"
entry .yvst.lim_max -width 10 -relief sunken -bd 2 -textvariable yvst_lim_max

button .yvst.plot -text "y vs theta" -command {upvar 0 yvstlim lim; \
	upvar 0 yvst_lim_min mn;\
	upvar 0 yvst_lim_max mx;\
	set strng "set xlabel \"theta \[radian\]\"\nset ylabel \"y \[m\]\"\n";\
	exec <@stdin >@stdout echo $strng;\
	plot1vs2varlines weak.dat 1 5 $lim $mn $mx}

pack .yvst.plot .yvst.lim .yvst.lim_min .yvst.to .yvst.lim_max -side left -fill x
#---------------------------------------
frame .xvsy
pack .xvsy -fill x

checkbutton .xvsy.lim -text limits -variable xvsylim -relief flat

entry .xvsy.lim_min -width 10 -relief sunken -bd 2 -textvariable xvsy_lim_min
label .xvsy.to -text "to"
entry .xvsy.lim_max -width 10 -relief sunken -bd 2 -textvariable xvsy_lim_max

button .xvsy.plot -text "y vs x" -command {upvar 0 xvsylim lim; \
	upvar 0 xvsy_lim_min mn;\
	upvar 0 xvsy_lim_max mx;\
	set strng "set xlabel \"x \[m\]\"\nset ylabel \"y \[m\]\"\n";\
	exec <@stdin >@stdout echo $strng;\
	plot1vs2var weak.dat 3 5 $lim $mn $mx}

pack .xvsy.plot .xvsy.lim .xvsy.lim_min .xvsy.to .xvsy.lim_max -side left -fill x
#---------------------------------------
frame .xyz
pack .xyz -fill x

checkbutton .xyz.lim -text rot_x -variable xyzlim -relief flat

entry .xyz.lim_min -width 10 -relief sunken -bd 2 -textvariable xyz_lim_min
label .xyz.to -text "rot_z"
entry .xyz.lim_max -width 10 -relief sunken -bd 2 -textvariable xyz_lim_max

button .xyz.plot -text "x,y vs theta" -command {upvar 0 xyzlim lim; \
	upvar 0 xyz_lim_min mn;\
	upvar 0 xyz_lim_max mx;\
	set strng "set xlabel \"turn\"\nset ylabel \"x \[m\]\"\nset zlabel \"y \[m\]\"\n";\
	exec <@stdin >@stdout echo $strng;\
	splott weak.dat 2 3 5 $lim $mn $mx}

pack .xyz.plot .xyz.lim .xyz.lim_min .xyz.to .xyz.lim_max -side left -fill x
#---------------------------------------
frame .rty
pack .rty -fill x

checkbutton .rty.lim -text rot_x -variable rtylim -relief flat

entry .rty.lim_min -width 10 -relief sunken -bd 2 -textvariable rty_lim_min
label .rty.to -text "rot_z"
entry .rty.lim_max -width 10 -relief sunken -bd 2 -textvariable rty_lim_max

button .rty.plot -text "orbit" -command {upvar 0 rtylim lim; \
	upvar 0 rty_lim_min mn;\
	upvar 0 rty_lim_max mx;\
	set strng "set xlabel \"east\"\nset ylabel \"north \[m\]\"\nset zlabel \"up \[m\]\"\n";\
	exec <@stdin >@stdout echo $strng;\
	splotp weak.dat (($rho+\$3)*cos(\$1)) (($rho+\$3)*sin(\$1)) 5 $lim $mn $mx}

pack .rty.plot .rty.lim .rty.lim_min .rty.to .rty.lim_max -side left -fill x
#---------------------------------------
frame .calc
pack .calc -fill x
button .calc.doit -text "calculate" -command { upvar 0 Q0 q0
    upvar 0 rho rhoo
    upvar 0 n nn
    upvar 0 nsteps nstepss
    upvar 0 x0 x00
    upvar 0 xp0 xp00
    upvar 0 y0 y00
    upvar 0 yp0 yp00
    upvar 0 z0 z00
    upvar 0 delta0 delta00
    upvar 0 nturns nturns0
    upvar 0 xoff xoff0
calculate $rhoo $nn $nstepss $x00 $xp00 $y00 $yp00 $z00 $delta00 $nturns0
}

pack .calc.doit -fill x
#---------------------------------------
frame .rho
pack .rho -fill x
label .rho.name -text "rho" -width 10
entry .rho.val -width 10 -relief sunken -bd 2 -textvariable rho

pack .rho.name .rho.val -side left -fill x
#---------------------------------------
frame .n
pack .n -fill x
label .n.name -text "n" -width 10
entry .n.val -width 10 -relief sunken -bd 2 -textvariable n

pack .n.name .n.val -side left -fill x
#---------------------------------------
frame .nsteps
pack .nsteps -fill x
label .nsteps.name -text "\#steps/turn" -width 10
entry .nsteps.val -width 10 -relief sunken -bd 2 -textvariable nsteps

pack .nsteps.name .nsteps.val -side left -fill x
#---------------------------------------
frame .nturns
pack .nturns -fill x
label .nturns.name -text "\#turns" -width 10
entry .nturns.val -width 10 -relief sunken -bd 2 -textvariable nturns

pack .nturns.name .nturns.val -side left -fill x
#---------------------------------------
frame .x0
pack .x0 -fill x
label .x0.name -text "x0" -width 10
entry .x0.val -width 10 -relief sunken -bd 2 -textvariable x0

pack .x0.name .x0.val -side left -fill x
#---------------------------------------
frame .xp0
pack .xp0 -fill x
label .xp0.name -text "xp0" -width 10
entry .xp0.val -width 10 -relief sunken -bd 2 -textvariable xp0

pack .xp0.name .xp0.val -side left -fill x
#---------------------------------------
frame .y0
pack .y0 -fill x
label .y0.name -text "y0" -width 10
entry .y0.val -width 10 -relief sunken -bd 2 -textvariable y0

pack .y0.name .y0.val -side left -fill x
#---------------------------------------
frame .yp0
pack .yp0 -fill x
label .yp0.name -text "yp0" -width 10
entry .yp0.val -width 10 -relief sunken -bd 2 -textvariable yp0

pack .yp0.name .yp0.val -side left -fill x
#---------------------------------------
frame .z0
pack .z0 -fill x
label .z0.name -text "z0" -width 10
entry .z0.val -width 10 -relief sunken -bd 2 -textvariable z0

#pack .z0.name .z0.val -side left -fill x
#---------------------------------------
frame .delta0
pack .delta0 -fill x
label .delta0.name -text "delta0" -width 10
entry .delta0.val -width 10 -relief sunken -bd 2 -textvariable delta0

pack .delta0.name .delta0.val -side left -fill x
#---------------------------------------
button .ps -text "PostScript of last plot" -command \
        {exec <@stdin >@stdout echo {set term postscript eps color "Times-Roman" 20};\
	exec <@stdin >@stdout echo {set output "weak.eps"};\
	exec <@stdin >@stdout echo {replot};\
	exec <@stdin >@stdout echo {set term x11}}
pack .ps -fill x
#---------------------------------------
button .quit -text "quit" -command exit
pack .quit -fill x

.xvst.plot configure -bg SeaGreen -fg yellow
.yvst.plot configure -bg SeaGreen -fg yellow
.xvsy.plot configure -bg SeaGreen -fg yellow
.xyz.plot configure -bg SeaGreen -fg yellow
.rty.plot configure -bg SeaGreen -fg yellow
.calc.doit configure -bg Gold -fg Red4
.ps configure -bg Blue -fg cyan
.quit configure -bg Red4 -fg yellow


