/* $Header: $ */
/* arr_of_2dgauss.c:  Generates a 2d Gaussian distribution */

#include <stdlib.h>
#include <math.h>

#include "randists.h"

/* The distribution, normalized to 1, is given by:
**
** f(x,y)= exp(-(beta*y^2 + 2*alpha*x*y + gamma*x^2)/2*epsilon)/(2*pi*epsilon)
**
** where epsilon beta is assumed to be positive, gamma=(1+alpha^2)/beta is also
** positive, alpha is any real number.
*/

#ifdef __SUN5
#define RAND01 drand48()
#else
#define RAND01 (double)random()/ranmax
#endif

int arr_of_2dgauss(int npts,        /* number of points to generate */
                   double beta,     /* beta (Twiss parameter) */
		   double alpha,    /* alpha = - beta'/2 (Twiss parameter) */
		   double epsilon,  /* rms emittance */
		   struct pt2d *arr /* array of points */
		   )
{
  int i;
  double eta, theta, ranmax;
  double r, sqrtbeta;
  double c, s;
  double twopi = 2.0*acos(-1.0);

  sqrtbeta = sqrt(beta);
#ifndef __SUN5
  ranmax = (double)RAND_MAX;
#endif
  for(i=0;i<npts;i++)
    {
      eta = 1.0-RAND01;
      if(eta<=0 || eta>1.0)return -1;
      r = sqrt(-log(eta)*epsilon);
      theta = twopi*RAND01;
      c = cos(theta);
      arr[i].x = sqrtbeta*r*c;
      arr[i].y = (r/sqrtbeta)*(sin(theta)-alpha*c);
    }
  return 0;
}

/*======================*/
#ifdef MAIN
/* A test program for the function */
#include <stdio.h>

const int n = 200;

int main(int argc, char** argv)
{
  int i;
  struct pt2d arr[200];

  if(arr_of_2dgauss(n, 10., 2., 0.00001, arr))
    {
      fprintf(stderr, "arr_of_2dgauss() failed\n");
      return -1;
    }
  for(i=0;i<n;i++)
    {
      fprintf(stdout, "%f  %f\n", arr[i].x, arr[i].y);
    }
  return 0;
}
#endif
