// $Header: $
// Tfunc.cc

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <iostream>

#define Tfunc_lib_goober 1
#include <Tfunc.hxx>

// constructors and other functions for the Tfunc.h beamline classes.

const double c = 299792458.0;  // m/s

//=========================
// For Class Trajectory

Trajectory** MapTrajArr(double *p, int n)
{  // This maps an array of arrays of doubles to an array of Trajectory's
  int i;
  double *q = p;

  if(p==0 || n==0)
    {
      std::cerr<<"? MapTrajArr(p="<<p<<", n="<<n<<")\n";
      return (Trajectory**)0;
    }

  Trajectory** traj = new Trajectory*[n+1];

  for(i=0;i<n;i++)
    {
      traj[i] = new Trajectory(q);
      q +=TFDIM;
    }
  traj[n] = 0;
  return traj;
}
//  --  --  --  --  --  --
void UnMapTrajArr(Trajectory** traj)
{  // Undo the malloc
  int i = 0;

  while(traj[i])
    {
      delete traj[i++];
    }
  delete traj;
  traj = (Trajectory**)0;
}
//  --  --  --  --  --  --
void Trajectory::zero()
{
  int i;
  for(i=0;i<TFDIM;i++)v[i] = 0;
}
//  --  --  --  --  --  --
Trajectory::Trajectory(const Trajectory &t)
{
  int i;

  v = new double[TFDIM];
  ialloced = 1;

  for(i=0;i<TFDIM;i++)
    {
      v[i] = t.v[i];
    }
}
//  --  --  --  --  --  --
Trajectory &Trajectory::operator=(const Trajectory &vv)
{
  int i;

  for(i=0;i<TFDIM;i++)
    {
      v[i] = vv.v[i];
    }
  return *this;
}
//  --  --  --  --  --  --
Trajectory Trajectory::operator+(const Trajectory &o) const
{
  int i;
  Trajectory temp;

  for(i=0;i<TFDIM;i++)temp.v[i] = v[i] + o.v[i];
  return temp;
}
//  --  --  --  --  --  --
Trajectory Trajectory::operator-(const Trajectory &o) const
{
  int i;
  Trajectory temp;

  for(i=0;i<TFDIM;i++)temp.v[i] = v[i] - o.v[i];
  return temp;
}
//  --  --  --  --  --  --
void Trajectory::operator+=(const Trajectory &o)
{
  int i;

  for(i=0;i<TFDIM;i++)v[i] += o.v[i];
}
//  --  --  --  --  --  --
void Trajectory::operator-=(const Trajectory &o)
{
  int i;

  for(i=0;i<TFDIM;i++)v[i] -= o.v[i];
}
//  --  --  --  --  --  --
Trajectory Trajectory::operator*(double s) const
{
  int i;
  Trajectory temp;

  for(i=0;i<TFDIM;i++) temp.v[i] = v[i]*s;
  return temp;
}
//  --  --  --  --  --  --
std::ostream &operator<<(std::ostream & os, const Trajectory &v)
{
  int i;
  char numstr[50];

  for(i=0;i<TFDIM;i++)
    {
      sprintf(numstr, Tfunc_format.c_str(), v.v[i]);
      os << "( " << numstr << " )\n";
    }
  return os;
}

//=========================
// For Class Tfunc

Tfunc Tfunc::unit()
{
  int i, j;
  for(i=0;i<TFDIM;i++)
    {
      for(j=0;j<TFDIM;j++)
	{
	  if(i==j)t.m[i][j] = 1;
	  else t.m[i][j] = 0;
	}
    }
  v.zero();
  return *this;
}
//  --  --  --  --  --  --
void Tfunc::offset(const Trajectory &offset)
{ // Offset the element by "offset" for misalignments, etc.
  Trajectory tmp;
  int i, j;

  for(i=0;i<TFDIM;i++)
    {
      tmp.v[i] = 0.0;
      for(j=0;j<TFDIM;j++)
	{
	  tmp.v[i] += (*this).t.m[i][j]*offset.v[j];
	}
    }
  v = v + offset - tmp;
}  
//  --  --  --  --  --  --
Tfunc &Tfunc::operator=(const Tfunc &tf)
{
  int i,j;

  v = tf.v; // copy trajectory

  for(i=0;i<TFDIM;i++)
    {
      for(j=0;j<TFDIM;j++)
	{
	  t.m[i][j] = tf.t.m[i][j];
	}
    }
  return *this;
}
//  --  --  --  --  --  --
Tfunc Tfunc::operator*(const Tfunc &A) const
{
  int i,j,k;
  Tfunc C;

  for(i=0;i<TFDIM;i++)
    {
      C.v.v[i] = v.v[i];
      for(j=0;j<TFDIM;j++)
	{
	  C.t.m[i][j] = 0;
	  for(k=0;k<TFDIM;k++)
	    {
	      C.t.m[i][j] += t.m[i][k] * A.t.m[k][j];
	    }
	  C.v.v[i] += t.m[i][j]*A.v.v[j];
	}
    }
  return C;
}
//  --  --  --  --  --  --
std::ostream &operator<<(std::ostream & os, const Tfunc &t)
{
  int i,j;
  char numstr[50];

  for(i=0;i<TFDIM;i++)
    {
      os << "[ ";
      for(j=0;j<TFDIM;j++)
	{
	  sprintf(numstr, Tfunc_format.c_str(), t.t.m[i][j]);
	  os << numstr;
	}
      sprintf(numstr, Tfunc_format.c_str(), t.v.v[i]);
      os << " ]   ( " << numstr << " )\n";
    }
  return os;
}

//=========================
// For Class Srot

Srot::Srot(double angle, int idir)
{
  t.m[0][0] = cos(angle*idir);
  t.m[1][1] = t.m[0][0];
  t.m[2][2] = t.m[0][0];
  t.m[3][3] = t.m[0][0];
  t.m[0][2] = sin(angle*idir);
  t.m[1][3] = t.m[0][2];
  t.m[2][0] = -t.m[0][2];
  t.m[3][1] = t.m[2][0];
}

//========================
// For Class Quad

Quad::Quad(double length, double k, double tilt, int idir, double beta)
{
  if (k > 0)   // F quad
    {
      double k_q = sqrt(k);
      double a = k_q * length;
      t.m[0][0] = cos(a);
      t.m[0][1] = sin(a)/k_q;
      t.m[1][0] = - k_q * sin(a);
      t.m[1][1] = cos(a);
      t.m[2][2] = cosh(a);
      t.m[2][3] = sinh(a)/k_q;
      t.m[3][2] = k_q * sinh(a);
      t.m[3][3] = cosh(a);
    }
  else if (k < 0)   // D quad
    {
      double k_q = sqrt(-k);
      double a = k_q * length;
      t.m[0][0] = cosh(a);
      t.m[0][1] = sinh(a)/k_q;
      t.m[1][0] = k_q * sinh(a);
      t.m[1][1] = cosh(a);
      t.m[2][2] = cos(a);
      t.m[2][3] = sin(a)/k_q;
      t.m[3][2] = - k_q * sin(a);
      t.m[3][3] = cos(a);
    }
  else    // quad is off - a drift
    {
      t.m[0][0] = 1.0;
      t.m[0][1] = length;
      t.m[1][0] = 0.0;
      t.m[1][1] = 1.0;
      t.m[2][2] = 1.0;
      t.m[2][3] = length;
      t.m[3][2] = 0.0;
      t.m[3][3] = 1.0;
    }
  t.m[4][5] = length*(1.0-beta*beta);

  if(tilt != 0)
    {
//      Srot r1(-tilt, idir), r2(tilt, idir);
      Tfunc temp;

      temp = Srot(-tilt, idir) * (*this) * Srot(tilt, idir);
      int i,j;
      for(i=0;i<TFDIM;i++)
	{
	  for(j=0;j<TFDIM;j++)
	    {
	      t.m[i][j] = temp.t.m[i][j];
	    }
	}
    }
}

//=========================
// For Class Multipole
Multipole::Multipole(double k0, double k1,
		     double t0, double t1,
		     double tilt, int idir)
{
  t.m[1][0] = -k1;
  t.m[3][2] = k1;

  if(tilt != 0)
    {
      Tfunc temp;

      temp = Srot(-(tilt+t1), idir) * (*this) * Srot(tilt+t1, idir);
      int i,j;
      for(i=0;i<TFDIM;i++)
	{
	  for(j=0;j<TFDIM;j++)
	    {
	      t.m[i][j] = temp.t.m[i][j];
	    }
	}
    }
  v.v[1] = idir*k0*cos(t0);      // note: cos(t0*idir)=cos(t0)
  v.v[3] = idir*k0*sin(t0*idir);
}

//=========================
// For Class Sbend

Sbend::Sbend(double length, double _theta, double k1,
	     double _e1, double _e2, double tilt, int idir, double beta)
{
  double k_x;
  double rho, fieldindex;
  double rx, ry, cx, cy, sx, sy, fudgesignx, fudgesigny;
  double p;
  int i;
  double theta, e1, e2; // tilt gets fixed up later.

// fix up for backwards lattice.
  theta = _theta*idir;
  e1 = _e1*idir;
  e2 = _e2*idir;

  if(theta==0)
    {
      t.m[0][1] = length;
      t.m[2][3] = length;
      return;
    }

  rho = length/theta;
  fieldindex = -k1*rho*rho;

  if (fieldindex == 0) {
    k_x = length/theta;
    t.m[0][0] = cos(theta);
    t.m[0][1] = k_x * sin(theta);
    t.m[0][5] = k_x * (1. - cos(theta));
    t.m[1][0] = - sin(theta)/k_x;
    t.m[1][5] = sin(theta);
    t.m[2][3] = length;
    t.m[4][5] = - k_x * (theta - sin(theta));
  }
  else if (fieldindex == 1) {
    t.m[0][1] = length;
    t.m[1][5] = theta;
    t.m[2][2] = cos(theta);
    t.m[2][3] = rho*sin(theta);
    t.m[3][2] = -sin(theta)/rho;
    t.m[4][5] = -length*theta*theta;
  }
  else {
    if(fieldindex < 0) {
      ry = sqrt(-fieldindex);
      cy = cosh(ry*theta);
      sy = sinh(ry*theta);
      fudgesigny = 1.0;
    }
    else {
      ry = sqrt(fieldindex);
      cy = cos(ry*theta);
      sy = sin(ry*theta);
      fudgesigny = -1;
    }
    if(fieldindex < 1) {
      rx = sqrt(1.0-fieldindex);
      cx = cos(rx*theta);
      sx = sin(rx*theta);
      fudgesignx = -1.0;
    }
    else {
      rx = sqrt(fieldindex-1.0);
      cx = cosh(rx*theta);
      sx = sinh(rx*theta);
      fudgesignx = 1.0;
    }
    t.m[0][0] = cx;
    t.m[0][1] = rho*sx/rx;
    t.m[0][5] = -fudgesignx*rho*(1.0-cx)/rx/rx;
    t.m[1][0] = fudgesignx*rx*sx/rho;
    t.m[1][5] = sx/rx;
    t.m[2][2] = cy;
    t.m[2][3] = rho*sy/ry;
    t.m[3][2] = fudgesigny*ry*sy/rho;
    /* The t[4,5] term differs from MAD in that we assume v/c=1 for the beam
     *  I believe MAD  uses something more like:
     *   t[4][5] = -rho*(theta-sx/rx)/(1.0-fieldindex)/beta^2
     *             +length/(beta*gamma)^2
     *  where the default energy is 1GeV and default mass is 0.511keV/c^2
     */
    t.m[4][5] = -rho*(theta-sx/rx)/(1.0-fieldindex);
  }
  t.m[1][1] = t.m[0][0];
  t.m[3][3] = t.m[2][2];
  t.m[4][0] = - t.m[1][5];
  t.m[4][1] = - t.m[0][5];

  t.m[4][5] += length*(1-beta*beta);

  if(e1 != 0) {             /* adjust for an entry angle e1 */
    p = tan(e1)/rho;
    for (i=0; i<TFDIM; i++) {
      t.m[i][0] = t.m[i][0]+p*t.m[i][1];
      t.m[i][2] = t.m[i][2]-p*t.m[i][3];
    }
  }
  if(e2 != 0) {             /* adjust for an exit angle e1 */
    p = tan(e2)/rho;
    for (i=0; i<TFDIM; i++) {
      t.m[1][i] = t.m[1][i]+p*t.m[0][i];
      t.m[3][i] = t.m[3][i]-p*t.m[2][i];
    }
  }

  // Apply the TILT angle

  if(tilt != 0)
    {
      Tfunc temp;

      temp = Srot(-tilt, idir) * (*this) * Srot(tilt, idir);
      int i,j;
      for(i=0;i<TFDIM;i++)
	{
	  for(j=0;j<TFDIM;j++)
	    {
	      t.m[i][j] = temp.t.m[i][j];
	    }
	}
    }
}
//=========================
Cavity::Cavity(double V, double q, double beta, double mc2, double h,
	       double L, double phi0)
{
  double gamma = 1.0/sqrt(1.0-beta*beta);
  double omegarf = 2*acos(-1.0)*beta*c*h/L;
  t.m[5][4] = omegarf*q*V*cos(phi0)/(beta*beta*beta*gamma*mc2*c);
}
//=========================
Cavity::Cavity(double V, double q, double beta, double mc2, double omegarf,
	       double phi0)
{
  double gamma = 1.0/sqrt(1.0-beta*beta);
  t.m[5][4] = omegarf*q*V*cos(phi0)/(beta*beta*beta*gamma*mc2*c);
}
//=========================
Trajectory Tfunc::operator*(const Trajectory &x0)
{
  int i, j;
  Trajectory x1;

  for(i=0;i<TFDIM;i++)
    {
      x1.v[i] = v.v[i];
      for(j=0;j<TFDIM;j++)
	{
	  x1.v[i] += t.m[i][j]*x0.v[j];
	}
    }
  return x1;
}
//=========================
// For Class Solen

Solen::Solen(double length, double k, double beta)
{
  double r = 1/k;
  double phi = length*k;
  double cp = cos(phi);
  double sp = sin(phi);

  t.m[3][3] = t.m[2][2] = t.m[1][1] = t.m[0][0] = (1.0+cp)/2.0;
  t.m[2][3] = t.m[0][1] = r*sp;
  t.m[3][2] = t.m[1][0] = -sp/4.0/r;
  t.m[0][2] = t.m[1][3] = sp/2.;
  t.m[2][0] = t.m[3][1] = -t.m[0][2];
  t.m[0][3] = r*(1-cp);
  t.m[3][0] = (1-cp)/4/r;
  t.m[2][1] = -t.m[0][3];
  t.m[1][2] = -t.m[3][0];

  t.m[4][5] = length*(1.0-beta*beta);
}
