// $Header: /home/owl/mackay/accel/models/bl/include/RCS/Tfunc.hxx,v 1.2 2000/07/04 21:11:58 mackay Exp mackay $
// Tfunc.h

#ifndef Tfunc_h
#define Tfunc_h 1

#include <stdlib.h>
#include <string>
// Note: I added the following to get the inverse function.
//       The TFunc class could probably could use a bit of a rewrite to use
//       the mat6 class more effectively.
#include "mat6/mat6.hxx"
const int TFDIM = MAT6DIM;

#ifndef Tfunc_lib_goober
// Format for << operator for Trajectory and Tfunc elements.
extern std::string Tfunc_format;
#else
std::string Tfunc_format = " %11.6lf";
#endif

//=========================

class Trajectory  // a TFDIM vector particle trajectory coordinate or
{		  // magnet offset.
private:
  int ialloced;
public:
  double *v;

//-- -- -- -- -- -- --

// The class constructors and destructors:
  Trajectory(double *p = 0)
    {
      if(p==0)
	{
	  v = new double[TFDIM];
	  ialloced = 1;
	} else {
	  v = p;
	  ialloced = 0;
	}
      zero();
    }

  ~Trajectory()
    {
      if(ialloced && v)
	{
	  delete [] v;
	  v = 0;
	  ialloced = 0;
	}
    }

// Copy constructor
  Trajectory(const Trajectory &);

// Member functions, etc.

  void zero();

  Trajectory &operator=(const Trajectory &vv);

  Trajectory operator+(const Trajectory &v) const;
  Trajectory operator-(const Trajectory &v) const;
  void operator+=(const Trajectory &v);
  void operator-=(const Trajectory &v);

  Trajectory operator*(double s) const;
  friend Trajectory operator*(double s, const Trajectory &v);

  friend Trajectory** MapTrajArr(double *p, int n);
  friend void UnMapTrajArr(Trajectory** traj);

  friend std::ostream &operator<<(std::ostream & os, const Trajectory &v);
};

inline Trajectory operator*(double s, const Trajectory &v)  { return v*s; }

//=========================

class Tfunc  // a TFDIMxTFDIM transfer matrix class
{
public:
  //  double t[TFDIM][TFDIM];
  mat6 t;
  Trajectory v;

//-- -- -- -- -- -- --

// The class constructors:
  Tfunc()
    {
      unit();
    }

// Member functions, etc.
  Tfunc unit();
  void offset(const Trajectory &offset);
  //  Tfunc &Tfunc::operator=(const Tfunc &tf);
  Tfunc &operator=(const Tfunc &tf);
  Tfunc operator*(const Tfunc &A) const;
  Trajectory operator*(const Trajectory &x0);

  friend std::ostream &operator<<(std::ostream & os, const Tfunc &t);
};

//=========================

class Drift: public Tfunc
{
public:

  Drift(double l, double beta = 1.0)
    {
      t.m[0][1] = l;
      t.m[2][3] = l;
      t.m[4][5] = l*(1.0-beta*beta);
    }
};

//=========================

class Quad: public Tfunc
{
public:

  Quad(double length, double k, double tilt=0, int idir=1, double beta=1.0);
};

//=========================

class Multipole: public Tfunc
{
public:

  Multipole(double k0,     // dipole strength
	    double k1,     // quadrupole strength
	    double t0=0,   // rotation angle: dipole component (=0 for normal)
	    double t1=0,   // rotation angle: quad component (=0 for normal)
	    double tilt=0, // overall tilt (Srot)
	    int idir=1     // direction of lattice (=1: forward; =-1: backward)
	    );
};

//=========================

class Srot: public Tfunc
{
public:

  Srot(double angle,  // rotation of (x, y) axes about s-coord
       int idir=1     // direction of lattice (=1: forward; =-1: backward)
       );
};

//=========================

class Sbend: public Tfunc
{
public:

  Sbend(double length,  // magnet length
	double theta,   // bend angle
	double k1=0,    // gradient
	double e1=0,    // input wedge angle
	double e2=0,    // output wedge angle
	double tilt=0,  // rotation (Srot) angle
	int idir=1,     // direction of lattice (=1: forward; =-1: backward)
	double beta=1.0 // v/c
	);
};

//=========================

class Cavity: public Tfunc
{
public:

  Cavity(double V,         // voltage [V]
	 double q,         // charge in units of e
	 double beta,      // v/c
	 double mc2,	   // rest mass of beam particle in [eV]
	 double h,         // harmonic number
	 double L,         // ring circumference  [m]
	 double phi0);     // synchronous phase   [rad]

  Cavity(double V,         // voltage [V]
	 double q,         // charge in units of e
	 double beta,      // v/c
	 double mc2,	   // rest mass of beam particle in [eV]
	 double omegarf,   // 2*pi*f_rf  (rf frequency of cavity) [rad/s]
	 double phi0);     // synchronous phase   [rad]
};

#endif

//=========================
class Solen: public Tfunc
{
public:

	Solen(double length, double k, double beta=1.0);
};
