// $Header: /home/ra/ride/rap/lambda/Geom3d/RCS/Plane_test.cxx,v 1.1 1996/02/23 02:02:08 mackay Exp mackay $
// This program tests some of the Plane class functions defined in Geom3d.h,

#include <Geom3d.hxx>

//==============================
int main(int argc, char **argv)
{
  std::cout.precision(8);
  std::cout.setf(std::ios::fixed);

  std::cout << "Test Plane constructors\n";
  Point O(0,0,0);
  Point c1(1,1,1);
  Point c2(1,1,0);
  std::cout << " O:\t" << O  << '\n';
  std::cout << "c1:\t" << c1 << '\n';
  std::cout << "c2:\t" << c2 << '\n';

// Plane constructors
  Plane m0;
  std::cout << "default plane: " << m0 << '\n';

  Plane m1(c1,0);
  std::cout << "m1(c1,0) = " << m1 << '\n';

  Plane m2(c1.x,c1.y,c1.z,1);
  std::cout << "m2(c1.x,c1.y,c1.z,1) = " << m2 << '\n';

  Plane m3(O,c1,c2);
  std::cout << "m3(O,c1,c2) = " << m3 << '\n';

  Line l1(O,c2);
  std::cout << "l1(origin --> c1): " << l1 << '\n';
  Plane m4(l1,c1);
  std::cout << "m4(l1,c2):" << m4 << '\n';
  Plane m5(c1,l1);
  std::cout << "m5(c1,l1):" << m5 << '\n';

// component access functions

  std::cout << "m5.cosines() = " << m5.cosines() << '\n';
  std::cout << "m5.r(): " << m5.r() << '\n';

  m5.Setcosines(c1);
  std::cout << "m5.Setcosines(c1):" << m5 << '\n';
  m5.Setr(10);
  std::cout << "m5.Setr(10):" << m5 << '\n';


// translation operations
  std::cout << "m5+c1: " << m5+c1 << '\n';
  std::cout << "c1+m5: " << c1+m5 << '\n';
  std::cout << "m5-c1: " << m5-c1 << '\n';
  std::cout << "c1-m5: " << c1-m5 << '\n';

  m5 += c1;
  std::cout << "m5+=c1: " << m5 << '\n';
  m5 -= c1;
  std::cout << "m5-=c1: " << m5 << '\n';

  std::cout << "-m5: " << -m5 << '\n';

// || and perp
  std::cout << "(m1 || m1):" << (m1 || m1)
       << "\t(m1 || m2):" << (m1 || m2) << '\n';
  std::cout << "perp(m1,m1):" << perp(m1,m1)
       << "\tperp(m1,m2):" << perp(m1,m2) << '\n';


//  let\'s see if the compiler complains
  int j = (l1 || m1) + (m1 || l1);

// ?? missing ??  j = perp(l1,m1) + perp(m1,l1);

// distances
  m1 = Plane(Point(1,1,0),0.5);
  std::cout << "c1: " << c1 << '\n';
  std::cout << "m1: " << m1 << '\n';
  std::cout << "distance(m1,c1): " << distance(m1,c1) << '\n';
  std::cout << "distance(c1,m1): " << distance(c1,m1) << '\n';
  m2 = m1 + c1;
  std::cout << "m2: " << m2 <<'\n';
  std::cout << "distance(m1,m2): " << distance(m1,m2) << '\n';

  m1 = Plane(1,0,0,0);
  std::cout << "m1: " << m1 << '\n';
  m2 = m1 + c1;
  std::cout << "m2: " << m2 <<'\n';
  std::cout << "distance(m1,m2): " << distance(m1,m2) << '\n';

// project a line onto a plane  (points done later with fitplane)
  std::cout << "m1.project(l1): " << m1.project(l1) << '\n';

// angles and rotations
  std::cout << "angle(m1,m2): " << angle(m1,m2) << '\n';
  std::cout << "angle(l1,m1): " << angle(l1,m1) << '\n';
  std::cout << "angle(m1,l1): " << angle(m1,l1) << '\n';

  double Piover4 = Piover2/2;

  m1 = Plane(Point(1,1,1),1);
  std::cout << "m1: " << m1 << '\n';

  m1 = rotate_x(m1,Piover4);
  std::cout << "m1=rotate_x(m1,Pi/4): "  << m1 << '\n';

  m1 = rotate_z(m1,-Piover2);
  std::cout << "m1=rotate_z(m1,-Pi/2): " << m1 << '\n';

  m1 = rotate_y(m1,Piover4);
  std::cout << "m1=rotate_y(m1,Pi/4): "  << m1 << '\n';

  m1 = rotate_z(m1,Piover2);
  std::cout << "m1=rotate_z(m1,Pi/2): "  << m1 << '\n';

// fit a plane to a bunch of points
  long i;
  const long npts = 10;
  Point q[npts];
  double ang = 0;
  double dang = 2*Pi/npts;
  double tilt = Pi/6;
  for(i=0;i<npts;i++)
    {
      Point temp(5,sin(ang),cos(ang));
      q[i] = rotate_y(temp,tilt);
      ang += dang;
    }
  Plane mfit = fitplane(q,npts);
  std::cout << "\nFit a plane through the following points:\n";
  for(i=0;i<npts;i++)
    {
      std::cout << i << ":\t" << q[i] << "\tresid:"
	   << q[i]-mfit.project(q[i]) << '\n';
    }
  std::cout << "The fitted plane is:" << mfit << '\n';

// Intersections


  return 0;
}
