#include <string>
#include <iostream>
#include <fstream>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "mat6/mat6.hxx"

#define DEBUGGERY 0
//=========
void xxnrerror(const char error_text[])
{
/*      void exit();   */

        fprintf(stderr,"Numerical Recipes run-time error...\n");
        fprintf(stderr,"%s\n",error_text);
        fprintf(stderr,"...now exiting to system...\n");
}
//--------------------------
int *xxivector(int nl,int nh)
{
        int *v;

        v=(int *)malloc((unsigned) (nh-nl+1)*sizeof(int));
        if (!v)
	  {
	    xxnrerror("allocation failure in xxivector()");
	    exit(-1);
	  }
        return v-nl;
}
//--------------------------
void free_xxivector(int *v, int nl, int nh)
{
        free((char*) (v+nl));
}
//--------------------------
double **xxdmatrix(int nrl, int nrh, int ncl, int nch)
{
        int i;
        double **m;

        m=(double **) malloc((unsigned) (nrh-nrl+1)*sizeof(double*));
        if (!m) 
	  {
	    xxnrerror("allocation failure 1 in xxdmatrix()");
	    exit(-1);
	  }
        m -= nrl;

        for(i=nrl;i<=nrh;i++) {
                m[i]=(double *) malloc((unsigned) (nch-ncl+1)*sizeof(double));
                if (!m[i])
		  {
		    xxnrerror("allocation failure 2 in xxdmatrix()");
		    exit(-2);
		  }
                m[i] -= ncl;
        }
        return m;
}
//--------------------------
void free_xxdmatrix(double **m, int nrl, int nrh, int ncl, int nch)
{
        int i;

        for(i=nrh;i>=nrl;i--) free((char*) (m[i]+ncl));
        free((char*) (m+nrl));
}
//--------------------------
#define SWAP(a,b) {double temp=(a);(a)=(b);(b)=temp;}

int dgaussj(double **a, int n, double **b, int m)
{
  int *indxc,*indxr,*ipiv;
  int i,icol,irow,j,k,l,ll,*xxivector(int, int);
  double big,dum,pivinv;
  void xxnrerror(const char []), free_xxivector(int*, int, int);

  if(DEBUGGERY)
    {
      printf("input matrix to dgaussj\n");
      for(i=1;i<=n;i++)
	{
	  printf("a:");
	  for(j=1;j<=n;j++)
	    {
	      printf("\t%e", a[i][j]);
	    }
	  printf("\n");
	}
    }

  indxc=xxivector(1,n);
  indxr=xxivector(1,n);
  ipiv=xxivector(1,n);
  for (j=1;j<=n;j++) ipiv[j]=0;
  for (i=1;i<=n;i++)
    {
      big=0.0;
      for (j=1;j<=n;j++)
	if (ipiv[j] != 1)
	  for (k=1;k<=n;k++)
	    {
	      if (ipiv[k] == 0)
		{
		  if (fabs(a[j][k]) >= big)
		    {
		      big=fabs(a[j][k]);
		      irow=j;
		      icol=k;
		    }
		}
	      else if (ipiv[k] > 1)
		{
		  xxnrerror("GAUSSJ: Singular Matrix-1");
		  return -1;
		}

	    }
      ++(ipiv[icol]);
      if (irow != icol)
	{
	  for (l=1;l<=n;l++) SWAP(a[irow][l],a[icol][l]);
          for (l=1;l<=m;l++) SWAP(b[irow][l],b[icol][l]);
        }
      indxr[i]=irow;
      indxc[i]=icol;
      if (a[icol][icol] == 0.0)
	{
	  xxnrerror("GAUSSJ: Singular Matrix-2");
	  return -2;
	}
      pivinv=1.0/a[icol][icol];
      a[icol][icol]=1.0;
      for (l=1;l<=n;l++) a[icol][l] *= pivinv;
      for (l=1;l<=m;l++) b[icol][l] *= pivinv;
      for (ll=1;ll<=n;ll++)
	if (ll != icol)
	  {
	    dum=a[ll][icol];
	    a[ll][icol]=0.0;
	    for (l=1;l<=n;l++) a[ll][l] -= a[icol][l]*dum;
	    for (l=1;l<=m;l++) b[ll][l] -= b[icol][l]*dum;
	  }
    }
  for (l=n;l>=1;l--)
    {
      if (indxr[l] != indxc[l])
	for (k=1;k<=n;k++) SWAP(a[k][indxr[l]],a[k][indxc[l]]);
    }
  if(DEBUGGERY)
    {
      printf("inverse of matrix from dgaussj\n");
      for(i=1;i<=n;i++)
	{
	  printf("o:");
	  for(j=1;j<=n;j++)
	    {
	      printf("\t%e", a[i][j]);
	    }
	  printf("\n");
	}
    }

  free_xxivector(ipiv,1,n);
  free_xxivector(indxr,1,n);
  free_xxivector(indxc,1,n);
  return 0;
}

#undef SWAP
//-------------------
mat6 inverse(const mat6 &M)
{
  mat6 Minv;
  int i, j;
  double **MM;
  double **t;
  MM = xxdmatrix(1, 6, 1, 6);
  t = xxdmatrix(1, 6, 1, 6);
  for(i=0;i<6;i++)
    for(j=0;j<6;j++)
      {
	MM[i+1][j+1] = M.m[i][j];
      }
  dgaussj(MM, 6, t, 0);
  for(i=0;i<6;i++)
    for(j=0;j<6;j++)
      {
	Minv.m[i][j] = MM[i+1][j+1];
      }
  free_xxdmatrix(MM, 1, 6, 1, 6);
  free_xxdmatrix(t, 1, 6, 1, 6);
  return Minv;
}
//-----------------
#ifdef MAIN

using namespace std;
int main(int argc, char** argv)
{
  mat6 M, N;

  M.m[1][2] = 10.;
  M.m[4][3] = 0.6;
  M.m[3][4] = -0.6;
  M.m[3][3] = M.m[4][4] = 0.8;
  cout << "M:\n" << M;
  N = inverse(M);
  cout << "N = inverse(M):\n" << N;
  cout << "N*M:\n" << N*M;

  return 0;
}
#endif
