#include <iostream>
#include <string>
#include <sstream>

#include <malloc.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "fodolat_gdef.inc"

const int MAXLEN = 400;

const double pi = acos(-1.0);
const double twopi = 2.0*pi;

const int NPAR = 8;

struct flattice
{
  std::string keyword;     // keyword MAD/SPINK char*4 element types
  std::string SWN;         // SiteWideName from NameLookup
  std::string swn;         // shortened SiteWideName without "-"'s
  std::string devname;     // DeviceName from NameLookup
  std::string latname;     // LatticeName from old MAD files
  std::string coordtype;   // CoordinateType from NameLookup
  double l;                // length of element [m]
  double sup;              // initial s-coordinate of element [m]
  double sdn;              // final s-coordinate of element [m]
  double par[NPAR];
} le;

char tab = '\t';
char cmsp[] = ", ";
char endl = '\n';
char whspc[] = ":\t, "; 

double stot = 0;
double theta;

FILE *fkey;
//==============
int process_cmd(int argc, char **argv)
{
  if(argc<2)
    {
      fprintf(stderr, "Usage: %s file.param\n", argv[0]);
      fprintf(stderr, " where file.param is the list of input parameters.\n");
      return -1;
    }
  GIfile = argv[1];

  GIinput();
  GIdump();

  return 0;
}
//------------
void printelt(void)
{
  printf("%s\t%s\t%s\t%s\t%s\t%s\t %f %f %f  %f %+f %+f 0 0 0 0 0\n",
	 le.keyword.c_str(),
	 le.SWN.c_str(),
	 le.swn.c_str(),
	 le.devname.c_str(),
	 le.latname.c_str(),
	 le.coordtype.c_str(),
	 le.l, le.sup, le.sdn,
	 le.par[0], le.par[1], le.par[2]);
}
//------------
void ADDQUAD(char df, int icell, double k)
{
  char str[MAXLEN];
  le.keyword = "QUAD";
  sprintf(str, "q%c%d", df, icell);
  le.SWN = le.swn = str;
  sprintf(str, "q%c", df);
  le.devname = str;
  //  le.latname = le.SWN;
  le.latname = le.devname;
  le.coordtype = "Traj";
  le.l = lquad;
  le.sup = stot;
  stot +=le.l;
  le.sdn = stot;
  if(df=='f')
    {
      le.par[1] = kf;
    }
  else
    {
      le.par[1] = kd;
    }
  le.par[0]=le.par[2]=le.par[3]=le.par[4]=le.par[5]=le.par[6]=le.par[7]=0;
  printelt();
  if(df=='f')
    {
      fprintf(fkey, "%f %f\n", le.sup, 0.0);
      fprintf(fkey, "%f %f\n", le.sup, yqf);
      fprintf(fkey, "%f %f\n", le.sdn, yqf);
      fprintf(fkey, "%f %f\n", le.sdn, 0.0);
    }
  else
    {
      fprintf(fkey, "%f %f\n", le.sup, 0.0);
      fprintf(fkey, "%f %f\n", le.sup, yqd);
      fprintf(fkey, "%f %f\n", le.sdn, yqd);
      fprintf(fkey, "%f %f\n", le.sdn, 0.0);
    }
}
//------------
void ADDBEND(char ab, int icell)
{
  char str[MAXLEN];
  le.keyword = "SBEN";
  sprintf(str, "b%c%d", ab, icell);
  le.SWN = le.swn = str;
  le.devname = "dh";
  le.latname = le.SWN;
  le.coordtype = "Traj";
  le.l = ldip;
  le.sup = stot;
  stot +=le.l;
  le.sdn = stot;
  le.par[0] = theta;
  le.par[1]=le.par[2]=le.par[3]=le.par[4]=le.par[5]=le.par[6]=le.par[7]=0;
  printelt();
  fprintf(fkey, "%f %f\n", le.sup, 0.0);
  fprintf(fkey, "%f %f\n", le.sup, yb);
  fprintf(fkey, "%f %f\n", le.sdn, yb);
  fprintf(fkey, "%f %f\n", le.sdn, 0.0);
}
//------------
void ADDSEXT(char df, int icell, double k)
{
  char str[MAXLEN];
  le.keyword = "SEXT";
  sprintf(str, "s%c%d", df, icell);
  le.SWN = le.swn = str;
  //      std::cout<<le.SWN<<"\n";
  le.devname = "sxf";
  le.latname = le.SWN;
  le.coordtype = "Traj";
  le.l = lsex;
  le.sup = stot;
  stot +=le.l;
  le.sdn = stot;
  if(df=='f')
    {
      le.par[2] = ksf;
    }
  else
    {
      le.par[2] = ksd;
    }
  le.par[0]=le.par[1]=le.par[3]=le.par[4]=le.par[5]=le.par[6]=le.par[7]=0;
  printelt();
  if(df=='f')
    {
      fprintf(fkey, "%f %f\n", le.sup, 0.0);
      fprintf(fkey, "%f %f\n", le.sup, ysf);
      fprintf(fkey, "%f %f\n", le.sdn, ysf);
      fprintf(fkey, "%f %f\n", le.sdn, 0.0);
    }
  else
    {
      fprintf(fkey, "%f %f\n", le.sup, 0.0);
      fprintf(fkey, "%f %f\n", le.sup, ysd);
      fprintf(fkey, "%f %f\n", le.sdn, ysd);
      fprintf(fkey, "%f %f\n", le.sdn, 0.0);
    }
}
//------------
void ADDRF(int icell, double freq)
{
  char str[MAXLEN];
  le.keyword = "RFCA";
  sprintf(str, "cavs%d", icell);
  le.SWN = le.swn = str;
  //      std::cout<<le.SWN<<"\n";
  le.devname = "cavs";
  le.latname = le.SWN;
  le.coordtype = "Traj";
  le.l = 0.0;  // thin lens rf cavity
  le.sup = stot;
  stot +=le.l;
  le.sdn = stot;
  le.par[4] = freq;
  le.par[0]=le.par[1]=le.par[2]=le.par[3]=le.par[5]=le.par[6]=le.par[7]=0;
  printelt();
}
//------------
void ADDTrim(char hv, int icell)
{
  char str[MAXLEN];
  sprintf(str, "t%c%d", hv, icell);
  le.SWN = le.swn = str;
  //      std::cout<<le.SWN<<"\n";
  if(hv=='h')
    {
      le.keyword = "HKIC";
      le.devname = "th";
    }
  else
    {
      le.keyword = "VKIC";
      le.devname = "tv";
    }
  le.latname = le.SWN;
  le.coordtype = "Traj";
  le.l = 0.0;  // thin lens steering corrector
  le.sup = stot;
  stot +=le.l;
  le.sdn = stot;
  le.par[4] = 0;
  le.par[0]=le.par[1]=le.par[2]=le.par[3]=le.par[5]=le.par[6]=le.par[7]=0;
  printelt();
  if(hv=='h')
    {
      fprintf(fkey, "%f %f\n", le.sup, 0.0);
      fprintf(fkey, "%f %f\n", le.sup, yth);
      fprintf(fkey, "%f %f\n", le.sdn, yth);
      fprintf(fkey, "%f %f\n", le.sdn, 0.0);
    }
  else
    {
      fprintf(fkey, "%f %f\n", le.sup, 0.0);
      fprintf(fkey, "%f %f\n", le.sup, ytv);
      fprintf(fkey, "%f %f\n", le.sdn, ytv);
      fprintf(fkey, "%f %f\n", le.sdn, 0.0);
    }
}
//------------
int main(int argc, char **argv)
{
  if(process_cmd(argc, argv)!=0) return -1;
  fprintf(stderr, "process_cmd = OK\n");

  // Build a FODO lattice with a cell of the form:  QF, SF, B, QD, SD, B
  double sup, sdn;
  char str[MAXLEN];

  int icell;

  fkey = fopen("fodolat.key", "w");

  theta = pi/(double)ncell; // dipole bend angle

  printf("# title: Simple FODO lattice for USPAS course.\n");
  printf("MARK start start mark start Traj 0 0 0  0 0 0 0 0 0 0 0\n");

  for(icell = 1;icell<=ncell;icell++)
    {
      ADDQUAD('f', icell, kf);
      stot += l0/2.0;
      ADDTrim('h', icell);
      stot += l0/2.0;
      ADDSEXT('f', icell, ksf);
      stot += l0;
      ADDBEND('a', icell);
      stot += l1;

      ADDQUAD('d', icell, kd);
      stot += l0/2.0;
      ADDTrim('v', icell);
      stot += l0/2.0;
      ADDSEXT('d', icell, ksd);
      stot += l0;
      ADDBEND('b', icell);
      if(icell==1)
	{
	  stot += l1/2.0;
	  ADDRF(icell, 0.0);
	  stot += l1/2.0;
	}
      else
	{
	  stot += l1;
	}
    }
  printf("MARK end end mark end Traj 0 %f %f 0 0 0 0 0 0 0 0\n", stot, stot);

  fclose(fkey);
  fkey = NULL;
  return 0;
}
